import ..syntax
import ..print_util

namespace sugar

mutual def expr.to_string, expr.to_string_list
with expr.to_string : expr → string
| (expr.val v) := v.to_string
| (expr.var x) := x
| (expr.op1 op e1) := op.to_string ++ " " ++ e1.to_string
| (expr.op2 op e1 e2) := e1.to_string ++ " " ++ op.to_string ++ " " ++ e2.to_string
| (expr.length e) := "len(" ++ e.to_string ++ ")"
| (expr.index e1 e2) := e1.to_string ++ "[" ++ e2.to_string ++ "]"
| (expr.call f args) := to_string f ++ "(" ++ (concat "," $ expr.to_string_list args) ++ ")"
| (expr.alloc n sz) := "alloc" ++ parens (expr.to_string n ++ ", " ++ expr.to_string sz)
with expr.to_string_list : list expr → list string
| [] := []
| (e :: es) := expr.to_string e :: expr.to_string_list es

instance expr.has_to_string : has_to_string expr :=
⟨ expr.to_string ⟩

def stmt.to_string_aux : stmt → list string
| (stmt.set x e) := [x ++ " = " ++ to_string e ++ ";"]
| (stmt.write x e1 e2) := [to_string x ++ "[" ++ to_string e1 ++ "]" ++ " = " ++ to_string e2 ++ ";"]
| (stmt.ifelse e p1 p2) := ["if " ++ parens (to_string e) ++ " {"] ++
  indent (stmt.to_string_aux p1) ++ ["} else {"] ++ indent (stmt.to_string_aux p2) ++ ["}"]
| (stmt.while g p1) := ["while " ++ parens (to_string g) ++ " {"] ++ indent (stmt.to_string_aux p1) ++ ["}"]
| (stmt.seq s1 s2) := stmt.to_string_aux s1 ++ stmt.to_string_aux s2
| (stmt.expr e) := [to_string e]
| (stmt.ret e) := ["return " ++ to_string e ++ ";"]
| (stmt.ifthen c s) := ["if " ++ parens (to_string c) ++ " {"] ++ indent (stmt.to_string_aux s) ++ ["}"]

def stmt.to_string (s : stmt) :=
concat "\n" (stmt.to_string_aux s)

instance stmt.has_to_string : has_to_string stmt :=
⟨ stmt.to_string ⟩

def func.to_string : func → string
| (func.mk name ps body) :=
  "def " ++ name ++ parens (string.join $ list.intersperse "," ps) ++ " " ++
   curlies (concat "\n" $ indent $ stmt.to_string_aux body)

instance func.has_to_string : has_to_string func :=
⟨ func.to_string ⟩

def prog.to_string : prog → string
| (prog.mk funcs main) :=
  concat "\n\n" $ list.map func.to_string funcs ++ [main]

instance prog.has_to_string : has_to_string prog :=
⟨ prog.to_string ⟩

end sugar
