import ..to_python

import ..to_python
import ..syntax
import .has_to_string

namespace sugar

mutual def expr.to_python, expr.to_python_list
with expr.to_python : expr → string
| (expr.val v) := to_string v
| (expr.var x) := to_string x
| (expr.op1 o e1) := "(" ++ to_string o ++ " " ++ expr.to_string e1 ++ ")"
| (expr.op2 o e1 e2) := "(" ++ expr.to_python e1 ++ " " ++ o.to_string ++ " " ++ expr.to_python e2 ++ ")"
| (expr.length e1) := "len(" ++ expr.to_python e1 ++ ")"
| (expr.index e1 e2) :=
    parens $ expr.to_python e1 ++ squares (expr.to_python e2)
| (expr.call f args) :=
  to_string f ++ (parens $ concat "," $ expr.to_python_list args)
| (expr.alloc i e) :=
  "(" ++ expr.to_python i ++ " * " ++ squares (expr.to_python e) ++ ")"
with expr.to_python_list : list expr → list string
| [] := []
| (e :: es) := expr.to_python e :: expr.to_python_list es

def stmt.to_python' : stmt → list string
| (stmt.set x e) := [to_string x ++ " = " ++ expr.to_python e]
| (stmt.write x e1 e2) := [to_string x ++ "[" ++ expr.to_python e1 ++ "]" ++ " = " ++ expr.to_python e2]
| (stmt.seq s1 s2) := stmt.to_python' s1 ++ stmt.to_python' s2
| (stmt.ifthen e p1) :=
  ["if " ++ expr.to_python e ++ ":"] ++ indent (stmt.to_python' p1)
| (stmt.ifelse e p1 p2) :=
  ["if " ++ expr.to_python e ++ ":"] ++
   indent (stmt.to_python' p1) ++ ["else:"] ++ indent (stmt.to_python' p2)
| (stmt.while e p1) :=
  ["while " ++ expr.to_python e ++ ":"] ++ indent (stmt.to_python' p1)
| (stmt.ret e) :=
  ["return " ++ e.to_python]
| (stmt.expr e) :=
  [e.to_python]

def stmt.to_python (s : stmt) :=
concat "\n" (stmt.to_python' s)

def func.to_python' : func → list string
| (func.mk name ps body) :=
  ("def " ++ name ++ parens (concat "," ps) ++ ":") :: indent (body.to_python')

def func.to_python (f : func) :=
concat "\n" (func.to_python' f)

instance func.has_to_python : has_to_python func :=
⟨ func.to_python ⟩

def prog.to_python' : prog → list string
| (prog.mk fs main) :=
  fs.map func.to_python ++ ["print " ++ main ++ "()"]

def prog.to_python (p : prog) : string :=
  concat "\n\n" (prog.to_python' p)

instance prog.has_to_python : has_to_python prog :=
⟨ prog.to_python ⟩

end sugar
