#!/usr/bin/env bash

TDIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
LEAN="lean --run ${TDIR}/../src/main.lean"

set -e

[[ $(type -P "lean") ]] || \
  { echo "ERROR: could not find lean, please ensure it is on your PATH first" 1>&2; exit 1; }

# ANSI color codes
BK=$'\033[1;30m' # black
RD=$'\033[1;31m' # red
GR=$'\033[1;32m' # green
YL=$'\033[1;33m' # yellow
BL=$'\033[1;34m' # blue
MG=$'\033[1;35m' # magenta
CY=$'\033[1;36m' # cyan
WT=$'\033[1;37m' # white
NC=$'\033[0m'    # no color

PASS="PASS"
FAIL="FAIL"

# only output color for ttys
if [ -t 1 ]; then
  PASS="${GR}PASS${NC}"
  FAIL="${RD}FAIL${NC}"
fi

function rand_bool {
  if [ "$(expr $RANDOM % 2)" -eq 0 ]; then
    echo "True"
  else
    echo "False"
  fi
}

function rand_int {
  if [ "$(expr $RANDOM % 2)" -eq 0 ]; then
    printf "-"
  fi
  expr $RANDOM % 100
}

function rand_str {
	env LC_CTYPE=C \
		tr -dc [:graph:] < /dev/urandom \
      | head -c $(expr $RANDOM % 1000)
}

function get_input {
  case $1 in
    "fact-rec.simp")
      echo $(rand_int)
      ;;
    "fact.simp")
      echo $(rand_int)
      ;;
    "fib.simp")
      echo $(rand_int)
      ;;
    "fib-rec.simp")
      echo $(rand_int)
      ;;
    "hello.simp")
      ;;
    "ispalindrome.simp")
      echo $(rand_str)
      ;;
    "minimal.simp")
      ;;
    "sort.simp")
      echo $(rand_int)
      ;;
    "srev.simp")
      echo $(rand_str)
      ;;
    "swap.simp")
      echo $(rand_int)
      echo $(rand_int)
      ;;
    "sum.simp")
      echo $(rand_int)
      ;;
    "partition.simp")
      echo $(rand_int)
      ;;
    "quicksort.simp")
      echo $(rand_int)
      ;;
    *)
      echo "ERROR: do not know how to set up input for $1" >&2
      exit 1
      ;;
  esac
}

function run_test {
  inp="$1.input"
  get_input $(basename "$1") > "$inp"

  spy="$1.sugar.py"
  if $LEAN --Spython "$1" > "$spy"; then
    if [ ! -s "$spy" ]; then
      echo "$spy is empty!"
      return 1
    fi
  else
    echo "Error: --Spython failed"
    return 1
  fi

  imp="$1.imp"
  if $LEAN --translate "$1" > "$imp"; then
    if [ ! -s "$imp" ]; then
      echo "$imp is empty!"
      return 1
    fi
  else
    echo "Error: --translate failed"
    return 1
  fi

  ipy="$1.imp.py"
  if $LEAN --Ipython "$imp" > "$ipy"; then
    if [ ! -s "$ipy" ]; then
      echo "$ipy is empty!"
      return 1
    fi
  else
    echo "Error: --Ipython failed"
    return 1
  fi

  sres="$1.s-result"
  if cat "$inp" | python "$spy" > "$sres"; then
    if [ ! -s "$sres" ]; then
      echo "$sres is empty!"
      return 1
    fi
  else
    echo "Error: running sugar python failed"
    return 1
  fi

  ires="$1.i-result"
  if cat "$inp" | python "$ipy" > "$ires"; then
    if [ ! -s "$ires" ]; then
      echo "$ires is empty!"
      return 1
    fi
  else
    echo "Error: running imp python failed"
    return 1
  fi

  cmp "$sres" "$ires"
}

for p in ${TDIR}/*.simp; do
  if run_test "$p" 2> /dev/null; then
    printf "%-20s" "$(basename "$p")"
    echo "$PASS"
  else
    printf "%-20s" "$(basename "$p")"
    echo "$FAIL"
  fi
  echo
done
